#include maps\mp\_utility;
#include maps\mp\_pier2_functions;
//made by rezil, do not modify!
/*
	Usage: maps\mp\_lockpicking::load();
	---
	Description: Loads the appropriate 2D HUD elements and localized strings,
				 this should be called right after you call maps\mp\_load::main();
	---
	Example: /
	
*/	
load()
{
	precacheShader("lockpicking_cylinder_0"); //blank
	precacheShader("lockpicking_cylinder_1"); //right-to-left progression 
	precacheShader("lockpicking_cylinder_2"); //of cylinder rotation
	precacheShader("lockpicking_cylinder_3");
	precacheShader("lockpicking_cylinder_4");
	precacheShader("lockpicking_cylinder_5");
	precacheShader("lockpicking_cylinder_6");
	precacheShader("lockpicking_cylinder_7");
	precacheShader("lockpicking_cylinder_8");
	precacheShader("lockpicking_cylinder_9");
	precacheShader("lockpicking_pin"); //the pin
	precacheShader("lockpicking_background"); //the background
	
	precacheString(&"LCKPCK_SUCCESS"); //"Success!"
	precacheString(&"LCKPCK_PICK_BROKE"); //"The lockpick broke!"
	precacheString(&"LCKPCK_NO_MORE_PICKS"); //"You have no lockpicks to pick the lock with!" <- maybe implement a lockpick count
	
	if(!isDefined(level.lockpicking_doors))
		level.lockpicking_doors = [];
}

/*
	Usage: <trigger> init(<door>, <unlock_origin>);
	---
	Description: Initializes the trigger to start the lockpicking procedure when the user triggers it
	
				 <trigger>: The trigger entity to call this function on
				 <door>: The door brushmodel to open when the lock is picked
				 <unlock_origin>: The origin to which the unlocked door will move and rotate to
	---
	Example: trig init(d, org);
	
*/

init(locked_door, unlock_origin)
{
	size = level.lockpicking_doors.size;
	if(!isDefined(level.lockpicking_doors[size]))
	{
		level.lockpicking_doors[size] = spawnStruct();
		level.lockpicking_doors[size].trigger = self;
		level.lockpicking_doors[size].door = locked_door;
		level.lockpicking_doors[size].org = unlock_origin;
	}
	level.lockpicking_doors[size] thread lockpickTriggerThink(unlock_origin[0]);
	
}
//Do not call these yourself unless you know what you're doing, the init function is the proper way of using this module
lockpickTriggerThink(rotateTo)
{
	while(true)
	{
		self.trigger waittill("trigger", user);
		if(user.hasLockpicks)
		{
			user thread holsterWeapons("holster");
			self.trigger triggerOff();
			success = user startLockpicking(); 
			user thread holsterWeapons("retrieve");
			//incidentally startLockpicking() blocks this function, 
			//so basically nobody else can use this trigger while the user is lockpicking
			if(success==1)
			{
				user iprintln(&"LCKPCK_SUCCESS");
				self.door playsound("pier2_open_door");
				self.door rotateyaw(rotateTo, 1, 0.5, 0.5);
				wait 4;
				self.door rotateyaw(rotateTo*(-1), 1, 0.5, 0.5);
				wait 1;
				self.door playsound("pier2_close_door");
			}
			else if(success==0)
				user iprintln(&"LCKPCK_PICK_BROKE");
				
			self.trigger triggerOn();
		} else iprintln(&"RZLPIER2_PRINTS_YOU_DONT_HAVE_ANY_LOCKPICKS");
	}
}
startLockpicking()
{
	NUM_OF_CYLINDERS = 3;
	SCALE = 20;
	STARTING_DIFF = 0.15; //waittime between turn frames
	
	success = -1; //invalid state of success, 0-fail, 1-success, 2-exit
	self.isLockpicking = true;
	self.cylinderLevel = 0; //on level 2 success = 1, else success = 0 and break lockpicking loop
	
	//self thread monitorUserStatus(); on user dies, on user goes to spec and other goodies
	self createLockpickHUDElements(NUM_OF_CYLINDERS, 0, 0, SCALE);
	self thread startLockpickingCylinder(SCALE, STARTING_DIFF, NUM_OF_CYLINDERS);
	
	self freezeControls(true);
	
	is_attack_pressed = false;
	while(self.isLockpicking)
	{
		if(self meleeButtonPressed())
			success = 2;
		else if(self attackButtonPressed() && !is_attack_pressed)
		{
			is_attack_pressed = true;
			if(self.currentTurn==4 || self.currentTurn==5 || self.currentTurn==6)
			{
				self.cylinderLevel++;
				if(self.cylinderLevel==NUM_OF_CYLINDERS) //max_level
					success = 1;
			}
			else success = 0;
		}
		else if(!self attackButtonPressed())
			is_attack_pressed = false;
		
		if(success!=-1) break;
		
		wait 0.05;
	}
	wait 0.05;
	self freezeControls(false);
	self stopLockpicking(NUM_OF_CYLINDERS);
	return success;
}
startLockpickingCylinder(scale, starting_diff, max_level)
{
	
	//20 turns as it rotates full circle, 10 turns you can see the hole, 10 you cannot
	//the window of opportunity is frames 4, 5 and 6
	self.currentTurn = 0;
	
	difficulty = starting_diff; //increases linearly by 0.05s
	currLevel = self.cylinderLevel;
	shader_name = "lockpicking_cylinder_0";
	
	while(self.isLockpicking)
	{
		//iprintln("self.cylinderLevel: "+self.cylinderLevel);
		if(currLevel<self.cylinderLevel)
		{
			//increase diff, set new level, move pin, TODO: set new currentTurn for this level!
			self.lockpickingHUD[1].y -= 40;
			wait 0.5; //padding, maybe remove this
			
			difficulty -= 0.05;
			if(difficulty<=0)
				difficulty = 0.05;
			currLevel = self.cylinderLevel;
			
			self.currentTurn = 0; //for now, will need to randomize turn frames
		}
		if(currLevel==max_level)
			break;
		
		if(self.currentTurn<=9)
			shader_name = "lockpicking_cylinder_"+self.currentTurn;
		else shader_name = "lockpicking_cylinder_0";
		
		self.lockpickingHUD[currLevel+2] setshader(shader_name, scale*4, scale);
		wait difficulty;
		
		self.currentTurn++;
		if(self.currentTurn>=20)
			self.currentTurn = 0;
	}
	
	if(self.isLockpicking)
		self stopLockpicking(max_level); //max_level == num_of_cylinders
}
createLockpickHUDElements(cylinder_num, x, y, scale)
{
	offset = 0; //offset increased by scale*2 every increment
	pin_offset = 60; //TODO: fine-tune this

	if(!isDefined(self.lockpickingHUD))
		self.lockpickingHUD = [];
		
		// Background
		if(!isDefined(self.lockpickingHUD[0]))
		{
			self.lockpickingHUD[0] = newClientHudElem(self);
			self.lockpickingHUD[0].alignX = "center";
			self.lockpickingHUD[0].alignY = "middle";
			self.lockpickingHUD[0].horzAlign = "center";
			self.lockpickingHUD[0].vertAlign = "middle";
			self.lockpickingHUD[0].sort = 1400;
		}
		self.lockpickingHUD[0].x = x;
		self.lockpickingHUD[0].y = y;
		self.lockpickingHUD[0] setshader("lockpicking_background", scale*12, scale*12);
		
		// Pin HUD element
		if(!isDefined(self.lockpickingHUD[1]))
		{
			self.lockpickingHUD[1] = newClientHudElem(self);
			self.lockpickingHUD[1].alignX = "center";
			self.lockpickingHUD[1].alignY = "middle";
			self.lockpickingHUD[1].horzAlign = "center";
			self.lockpickingHUD[1].vertAlign = "middle";
			self.lockpickingHUD[1].sort = 1600;
			self.lockpickingHUD[1].foreground = 1;
		}
		self.lockpickingHUD[1].x = x;
		self.lockpickingHUD[1].y = y + pin_offset;
		self.lockpickingHUD[1] setshader("lockpicking_pin", scale, scale*4);
		
	for(i=2;i<cylinder_num+2;i++) //0 is background, 1 is pin
	{
		if(!isDefined(self.lockpickingHUD[i]))
		{
			self.lockpickingHUD[i] = newClientHudElem(self);
			self.lockpickingHUD[i].alignX = "center";
			self.lockpickingHUD[i].alignY = "middle";
			self.lockpickingHUD[i].horzAlign = "center";
			self.lockpickingHUD[i].vertAlign = "middle";
		}
		self.lockpickingHUD[i].alpha = 1;
		self.lockpickingHUD[i].x = x;
		self.lockpickingHUD[i].y = y - offset;
		self.lockpickingHUD[i].sort = 1500+i;
		self.lockpickingHUD[i] setshader("lockpicking_cylinder_0", scale*4, scale); //40 is approx. 64 units, TODO: randomize cylinder frame+turn frame(int)
		
		offset += scale*2;
	}
}
stopLockpicking(num_of_cylinders)
{
	self.isLockpicking = false;
	for(i=0;i<num_of_cylinders+2;i++)
		if(isDefined(self.lockpickingHUD[i]))
			self.lockpickingHUD[i] destroy();
	
	
}
/* UTILITY */
compareOrigin(ent1, ent2)
{
	if(ent1.origin[0]==ent2.origin[0] &&
	   ent1.origin[1]==ent2.origin[1] &&
	   ent1.origin[2]==ent2.origin[2])
		return true;
	else return false;
}

compareAngles(ent1, ent2)
{
	if(ent1.angles[0]==ent2.angles[0] &&
	   ent1.angles[1]==ent2.angles[1] &&
	   ent1.angles[2]==ent2.angles[2])
		return true;
	else return false;
}