//made by rezil, do not modify!
/*
The music manager is going to manage our music bits. This manager is local to each player, it's going to play
random exploration music whenever we're not in a cinematic, we're not currently playing a
bit of music and there's been at least 120-180(random, adjustable) seconds between each exploration music bit. 

After a music bit has been played, put it in an 'already played' array so we don't repeat ourselves during the game. If
we run out of music to play, the 'already played' array is emptied and the process is started over. Hopefully(if we have enough music bits)
this will lead to an interesting sound background while we're playing the map.

If a music bit is playing and we're entering a cutscene, the cutscene music takes precedence over the exploration music. Talking and
playing music should be in separate channels. 
*/

initMusicManager()
{
	self notify("music_manager_reset");
	
	level.EXPLORATION_MUSIC_BITS = 9; //will take at least 18 minutes to go through all of these
	self.musicBits = [];
	
	self.musicOrderIndex = 0;
	self.musicOrder = [];
	//exploration
	for(i=0;i<level.EXPLORATION_MUSIC_BITS;i++)
	{
		self.musicBits[i] = spawnStruct();
		self.musicBits[i].played = false; //reset all to 'not played yet'
		self.musicBits[i].name = "pier2_exploration_amb_"+(i+1);
		
		self.musicOrder[i] = i; //We'll shuffle these later
	}
	
	self.playingMusicBit = "none"; //which sound alias is being played
	self.playingUniqueMusic = false;
	
	self shuffleMusicBits();
	self thread mainMusicLoop();
}
mainMusicLoop()
{
	self endon("music_manager_reset");
	while(true)
	{
		wait 120;
		r = randomInt(60);
		wait r;
		
		if(!self.isInCinematic && self.playingMusicBit=="none" && !self.playingUniqueMusic)
			self thread playMusicBit();
	}
}
//plays a random exploration bit of music
playMusicBit()
{
	self endon("stopped_music_bits");
	
	random = self.musicOrder[self.musicOrderIndex];
	self.musicOrderIndex++;
	
	if(self.musicBits[random].played)
		return; //should never happen
	
	self.playingMusicBit = self.musicBits[random].name;
	self.musicBits[random].played = true;
	
	self playLocalSound(self.musicBits[random].name);
	
	switch(self.musicBits[random].name)
	{
		case "pier2_exploration_amb_1":
			wait 31;
			break;
		case "pier2_exploration_amb_2":
			wait 35;
			break;
		case "pier2_exploration_amb_3":
			wait 18;
			break;
		case "pier2_exploration_amb_4":
			wait 32;
			break;
		case "pier2_exploration_amb_5":
			wait 8;
			break;
		case "pier2_exploration_amb_6":
			wait 48;
			break;
		case "pier2_exploration_amb_7":
			wait 60;
			break;
		case "pier2_exploration_amb_8":
			wait 48;
			break;
		case "pier2_exploration_amb_9":
			wait 38;
			break;
	}
	self.playingMusicBit = "none";
	
	//after we're done playing, check if we have any bits left to play
	if(!self hasMusicBitsLeft())
	{
		//if we don't reset all music to 'not played'
		self resetMusicBits();
		//re-shuffle as well
		self shuffleMusicBits();
	}
}
//when this is called, it takes precedence over playMusicBit()
playUniqueMusicBit(soundalias)
{
	stopMusicBits();
	self.playingUniqueMusic = true;
	
	switch(soundalias)
	{
		case "ambience_music_bridge_raise":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 16;
			break;
		case "ambience_music_bunker":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 15;
			break;
		case "ambience_music_climbing_crane":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 16;
			break;
		case "ambience_music_speeder_dropped":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 55;
			break;
		case "ambience_music_bed":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 3;
			break;
		case "ambience_music_ending_credits":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 85;
			break;
		case "pier2vo_sfx_duke_mode_music":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 16;
			break;
		case "pier2_misc_secret_found":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 5;
			break;
		case "dark_coffee":
			self.playingMusicBit = soundalias;
			self playLocalSound(soundalias);
			wait 351;
			break;
	}
	self.playingMusicBit = "none";
	self.playingUniqueMusic = false;
}
stopMusicBits()
{
	self notify("stopped_music_bits");
	
	self playLocalSound("pier2_exploration_null");
	self.playingMusicBit = "none";
}
hasMusicBitsLeft()
{
	hasLeft = false;
	for(i=0;i<level.EXPLORATION_MUSIC_BITS;i++)
	{
		if(!self.musicBits[i].played)
			hasLeft = true; //if it's not played, set hasLeft to true
	}
	
	return hasLeft;
}
resetMusicBits()
{
	for(i=0;i<level.EXPLORATION_MUSIC_BITS;i++)
		self.musicBits[i].played = false; //reset all to 'not played yet'
}
//FisherYates shuffle
shuffleMusicBits()
{
	self.musicOrderIndex = 0; //restart array index
	
	for(i=self.musicOrder.size-1;i>0;i--)
	{
		j = randomIntRange(0, i+1);
		oldVal = self.musicOrder[i];
		self.musicOrder[i] = self.musicOrder[j];
		self.musicOrder[j] = oldVal;
	}
}
